@@ -26,6 +26,7 @@ is setup by creating a `setup.py` file in the project with the following::
     bundlebuilder.start()
 '''
 
+import six
 import argparse
 import operator
 import os
@@ -40,12 +41,13 @@ import gettext
 import logging
 from glob import glob
 from fnmatch import fnmatch
-from ConfigParser import ConfigParser
-import xml.etree.cElementTree as ET
-from HTMLParser import HTMLParser
+from six.moves.configparser import ConfigParser
+import xml.etree.ElementTree as ET
+from six.moves.html_parser import HTMLParser
 
 from sugar3 import env
 from sugar3.bundle.activitybundle import ActivityBundle
+from six.moves import reduce
 
 
 IGNORE_DIRS = ['dist', '.git', 'screenshots', 'debian', '.pc']
@@ -150,11 +152,11 @@ class Builder(object):
             args = ['msgfmt', '--output-file=%s' % mo_file, file_name]
             retcode = subprocess.call(args)
             if retcode:
-                print 'ERROR - msgfmt failed with return code %i.' % retcode
+                print('ERROR - msgfmt failed with return code %i.' % retcode)
                 if self._no_fail:
                     continue
 
-            cat = gettext.GNUTranslations(open(mo_file, 'r'))
+            cat = gettext.GNUTranslations(open(mo_file, 'rb'))
             translated_name = cat.gettext(self.config.activity_name)
             translated_summary = cat.gettext(self.config.summary)
             if translated_summary is None:
@@ -205,7 +207,7 @@ class Packager(object):
             elif stdout:
                 # pylint: disable=E1103
                 git_output = [path.strip() for path in
-                              stdout.strip('\n').split('\n')]
+                              stdout.decode().strip('\n').split('\n')]
                 files = []
                 for line in git_output:
                     ignore = False
@@ -278,7 +280,8 @@ class Installer(Packager):
         Packager.__init__(self, builder.config)
         self.builder = builder
 
-    def install(self, prefix, install_mime=True, install_desktop_file=True):
+    def install(self, destdir, prefix,
+                install_mime=True, install_desktop_file=True):
         self.builder.build()
 
         activity_path = os.path.join(prefix, 'share', 'sugar', 'activities',
@@ -288,21 +291,26 @@ class Installer(Packager):
 
         for f in self.get_files_in_git():
             source_path = os.path.join(self.config.source_dir, f)
-            dest_path = os.path.join(activity_path, f)
+            dest_path = os.path.join(destdir,
+                                     os.path.relpath(activity_path, '/'), f)
             source_to_dest[source_path] = dest_path
 
         for f in self.builder.get_locale_files():
             source_path = os.path.join(self.builder.locale_dir, f)
 
             if source_path.endswith(".mo"):
-                dest_path = os.path.join(prefix, 'share', 'locale', f)
+                dest_path = os.path.join(destdir,
+                                         os.path.relpath(prefix, '/'),
+                                         'share', 'locale', f)
             else:
-                dest_path = os.path.join(activity_path, 'locale', f)
+                dest_path = os.path.join(destdir,
+                                         os.path.relpath(activity_path, '/'),
+                                         'locale', f)
 
             source_to_dest[source_path] = dest_path
 
-        for source, dest in source_to_dest.items():
-            print 'Install %s to %s.' % (source, dest)
+        for source, dest in list(source_to_dest.items()):
+            print('Install %s to %s.' % (source, dest))
 
             path = os.path.dirname(dest)
             if not os.path.exists(path):
@@ -314,10 +322,10 @@ class Installer(Packager):
             self.config.bundle.install_mime_type(self.config.source_dir)
 
         if install_desktop_file:
-            self._install_desktop_file(prefix, activity_path)
-            self._generate_appdata(prefix, activity_path)
+            self._install_desktop_file(destdir, prefix, activity_path)
+            self._generate_appdata(destdir, prefix, activity_path)
 
-    def _install_desktop_file(self, prefix, activity_path):
+    def _install_desktop_file(self, destdir, prefix, activity_path):
         cp = ConfigParser()
         section = 'Desktop Entry'
         cp.add_section(section)
@@ -325,7 +333,9 @@ class Installer(Packager):
 
         # Get it from the activity.info for the non-translated version
         info = ConfigParser()
-        info.read(os.path.join(activity_path, 'activity', 'activity.info'))
+        info_path = os.path.join(destdir, os.path.relpath(activity_path, '/'),
+                                 'activity', 'activity.info')
+        info.read(info_path)
         cp.set(section, 'Name', info.get('Activity', 'name'))
         if info.has_option('Activity', 'summary'):
             cp.set(section, 'Comment', info.get('Activity', 'summary'))
@@ -351,26 +361,25 @@ class Installer(Packager):
         cp.set(section, 'Path', activity_path)  # Path == CWD for running
 
         name = '{}.activity.desktop'.format(self.config.bundle_id)
-        path = os.path.join(prefix, 'share', 'applications', name)
+        path = os.path.join(destdir, os.path.relpath(prefix, '/'),
+                            'share', 'applications', name)
         if not os.path.isdir(os.path.dirname(path)):
             os.makedirs(os.path.dirname(path))
         with open(path, 'w') as f:
             cp.write(f)
 
-    def _generate_appdata(self, prefix, activity_path):
+    def _generate_appdata(self, destdir, prefix, activity_path):
         info = ConfigParser()
-        info.read(os.path.join(activity_path, 'activity', 'activity.info'))
+        info_path = os.path.join(destdir, os.path.relpath(activity_path, '/'),
+                                 'activity', 'activity.info')
+        info.read(info_path)
 
         required_fields = ['metadata_license', 'license', 'name', 'icon',
                            'description']
         for name in required_fields:
             if not info.has_option('Activity', name):
-                print('[WARNING] Activity needs more metadata for AppStream '
-                      'file')
-                print('  Without an AppStream file, the activity will NOT '
-                      'show in software stores!')
-                print('  Please `pydoc sugar3.activity.bundlebuilder` for'
-                      'more info')
+                print('[WARNING] missing AppStream metadata, '
+                      'see `pydoc sugar3.bundle`')
                 return
 
         # See https://www.freedesktop.org/software/appstream/docs/
@@ -410,7 +419,8 @@ class Installer(Packager):
             ET.SubElement(root, 'url', type='bugtracker').text = \
                 info.get('Activity', 'repository')
 
-        path = os.path.join(prefix, 'share', 'metainfo',
+        path = os.path.join(destdir, os.path.relpath(prefix, '/'),
+                            'share', 'metainfo',
                             self.config.bundle_id + '.appdata.xml')
         if not os.path.isdir(os.path.dirname(path)):
             os.makedirs(os.path.dirname(path))
@@ -429,7 +439,7 @@ def cmd_check(config, options):
     if options.choice == 'integration':
         run_unit_test = False
 
-    print "Running Tests"
+    print("Running Tests")
 
     test_path = os.path.join(config.source_dir, "tests")
 
@@ -443,22 +453,22 @@ def cmd_check(config, options):
             all_tests = unittest.defaultTestLoader.discover(unit_test_path)
             unittest.TextTestRunner(verbosity=options.verbose).run(all_tests)
         elif not run_unit_test:
-            print "Not running unit tests"
+            print("Not running unit tests")
         else:
-            print 'No "unit" directory found.'
+            print('No "unit" directory found.')
 
         if os.path.isdir(integration_test_path) and run_integration_test:
             all_tests = unittest.defaultTestLoader.discover(
                 integration_test_path)
             unittest.TextTestRunner(verbosity=options.verbose).run(all_tests)
         elif not run_integration_test:
-            print "Not running integration tests"
+            print("Not running integration tests")
         else:
-            print 'No "integration" directory found.'
+            print('No "integration" directory found.')
 
-        print "Finished testing"
+        print("Finished testing")
     else:
-        print "Error: No tests/ directory"
+        print("Error: No tests/ directory")
 
 
 def cmd_dev(config, options):
@@ -472,9 +482,9 @@ def cmd_dev(config, options):
         os.symlink(config.source_dir, bundle_path)
     except OSError:
         if os.path.islink(bundle_path):
-            print 'ERROR - The bundle has been already setup for development.'
+            print('ERROR - The bundle has been already setup for development.')
         else:
-            print 'ERROR - A bundle with the same name is already installed.'
+            print('ERROR - A bundle with the same name is already installed.')
 
 
 def cmd_dist_xo(config, options):
@@ -490,9 +500,9 @@ def cmd_dist_xo(config, options):
 def cmd_fix_manifest(config, options):
     '''Add missing files to the manifest (OBSOLETE)'''
 
-    print 'WARNING: The fix_manifest command is obsolete.'
-    print '         The MANIFEST file is no longer used in bundles,'
-    print '         please remove it.'
+    print('WARNING: The fix_manifest command is obsolete.')
+    print('         The MANIFEST file is no longer used in bundles,')
+    print('         please remove it.')
 
 
 def cmd_dist_source(config, options):
@@ -506,7 +516,11 @@ def cmd_install(config, options):
     """Install the activity in the system"""
 
     installer = Installer(Builder(config))
-    installer.install(options.prefix, options.install_mime, options.install_desktop_file)
+    installer.install(
+        options.destdir,
+        options.prefix,
+        options.install_mime,
+        options.install_desktop_file)
 
 
 def _po_escape(string):
@@ -563,12 +577,15 @@ def cmd_genpot(config, options):
     f.close()
 
     args = ['xgettext', '--join-existing', '--language=Python',
-            '--keyword=_', '--add-comments=TRANS:', '--output=%s' % pot_file]
+            '--keyword=_', '--add-comments=TRANS:',
+            '--output=%s' % pot_file]
+    if six.PY3:
+        args.insert(5, '--from-code=UTF-8')
 
     args += python_files
     retcode = subprocess.call(args)
     if retcode:
-        print 'ERROR - xgettext failed with return code %i.' % retcode
+        print('ERROR - xgettext failed with return code %i.' % retcode)
 
 
 def cmd_build(config, options):
@@ -588,6 +605,9 @@ def start():
     install_parser.add_argument(
         "--prefix", dest="prefix", default=sys.prefix,
         help="Path for installing")
+    install_parser.add_argument(
+        "--destdir", dest="destdir", default="/",
+        help="Path for staged install")
     install_parser.add_argument(
         "--skip-install-mime", dest="install_mime",
         action="store_false", default=True,
@@ -603,7 +623,7 @@ def start():
                               choices=['unit', 'integration'],
                               help="run unit/integration test")
     check_parser.add_argument("--verbosity", "-v", dest="verbose",
-                              type=int, choices=range(0, 3),
+                              type=int, choices=list(range(0, 3)),
                               default=1, nargs='?',
                               help="verbosity for the unit tests")
 
@@ -625,9 +645,9 @@ def start():
     source_dir = os.path.abspath(os.path.dirname(sys.argv[0]))
     config = Config(source_dir)
 
-    try:
-        globals()['cmd_' + options.command](config, options)
-    except (KeyError, IndexError):
+    if 'cmd_' + (options.command or '') in globals():
+        globals()['cmd_' + (options.command or '')](config, options)
+    else:
         parser.print_help()
 
 
